import psycopg2
import vanna as vn
import logging
from cryptography.fernet import Fernet
import json

logging.basicConfig(level=logging.CRITICAL)

def decrypt_data(encrypted_data, key):
    cipher = Fernet(key)
    return cipher.decrypt(encrypted_data).decode()


def train_model(*args):
    kwargs, secert_key = args
    kwargs = json.loads(decrypt_data(kwargs, secert_key))
    vn_model_name = kwargs.get("vn_model_name")
    vn_api_key = kwargs.get("vn_api_key")
    model_name = kwargs.get("model_name")
    connection = kwargs.get("connection", False)
    dbname = kwargs.get("dbname", False)
    user = kwargs.get("user", False)
    pwd = kwargs.get("pwd", False)
    host = kwargs.get("host", False)
    port = kwargs.get("port", False)
    print(f"Training Started for {model_name}")
    try:
        connection = psycopg2.connect(
            dbname=dbname,
            user=user,
            password=pwd,
            host=host,
            port=port
        )

        def get_table_ddl(table_name, cur):
            cur.execute(f"""
                    SELECT column_name, data_type, is_nullable
                    FROM information_schema.columns
                    WHERE table_name = '{table_name}'
                    """)
            columns = cur.fetchall()

            cur.execute(f"""
                    SELECT
                        tc.table_name, kcu.column_name, ccu.table_name AS foreign_table_name,
                        ccu.column_name AS foreign_column_name
                    FROM
                        information_schema.table_constraints AS tc
                        JOIN information_schema.key_column_usage AS kcu
                            ON tc.constraint_name = kcu.constraint_name
                        JOIN information_schema.constraint_column_usage AS ccu
                            ON ccu.constraint_name = tc.constraint_name
                    WHERE
                        constraint_type = 'FOREIGN KEY' AND tc.table_name='{table_name}';
                    """)
            foreign_keys = cur.fetchall()

            ddl = f"CREATE TABLE IF NOT EXISTS public.{table_name}\n(\n"
            for column in columns:
                col_name, data_type, is_nullable = column
                ddl += f"    {col_name} {data_type} {'' if is_nullable == 'YES' else 'NOT '}NULL,\n"

            # Add foreign key constraints
            for fk in foreign_keys:
                table_name, column_name, foreign_table, foreign_column = fk
                ddl += f"    CONSTRAINT {table_name}_{column_name}_fkey FOREIGN KEY ({column_name})\n"
                ddl += f"        REFERENCES public.{foreign_table} ({foreign_column}) MATCH SIMPLE\n"
                ddl += "        ON UPDATE NO ACTION\n"
                ddl += "        ON DELETE SET NULL,\n"

            ddl += f"    CONSTRAINT {table_name}_pkey PRIMARY KEY (id)\n"

            ddl += ");"
            return ddl

        cursor = connection.cursor()

        vn.set_api_key(vn_api_key)
        vn_models = vn.get_models()
        if vn_model_name.lower() not in vn_models:
            vn.create_model(model=vn_model_name.lower(), db_type="Postgres")
        vn.set_model(vn_model_name.lower())
        cursor.execute(f"""SELECT id, table_name FROM ir_model WHERE model = '{model_name}';""")
        model_id = cursor.fetchall()

        if len(model_id):
            tabel_id, tables_name = model_id[0]
            cursor.execute("""
                UPDATE ai_train_info
                SET model_id = %s, is_active = %s, "table" = %s, state = %s
                WHERE model_name = %s
            """, (tabel_id, True, tables_name, 'done', model_name))
            connection.commit()
            print(f"Training Completed for {model_name}")
            ddl_val = get_table_ddl(tables_name, cursor)
            vn.add_ddl(ddl_val)
    except Exception as e:
        logging.error(f"An error occurred while training model {model_name}: {str(e)}")
        print(f"An error occurred while training model {model_name}: {str(e)}")
    finally:
        if connection:
            connection.close()
    