from datetime import timedelta
from redis import Redis
from rq import Queue
from cy_ai_trainer.model_trainer import train_model
import logging
import psutil
import subprocess
from cryptography.fernet import Fernet
import json

logging.basicConfig(level=logging.CRITICAL)


class TrainModels:
    def __init__(self,
                 models_list,
                 vn_model_name,
                 vn_api_key,
                 shell_path,
                 db_name=None,
                 user=None,
                 pwd=None,
                 host="localhost",
                 pg_port=5432,
                 redis_host="localhost",
                 redis_port=6379,
                 redis_db=0
                 ):
        self.models_list = models_list
        self.vn_model_name = vn_model_name
        self.vn_api_key = vn_api_key
        self.db_name = db_name
        self.user = user
        self.pwd = pwd
        self.host = host
        self.pg_port = pg_port
        self.redis_host = redis_host
        self.redis_port = redis_port
        self.redis_db = redis_db
        self.shell_path = shell_path

        if not self.db_name and not self.user and not self.pwd:
            raise Exception("Please provide db creds")
        if not self.is_rq_worker_running():
            command = ["rq", "worker", "--with-scheduler"]
            subprocess.run(command, check=True, cwd=shell_path)
        self.queue = Queue(connection=Redis(port=redis_port, host=redis_host, db=redis_db))

    def train_models(self, sec=2):
        secret_key = Fernet.generate_key()
        kwargs = {
            'vn_model_name': self.vn_model_name,
            'vn_api_key': self.vn_api_key,
            'connection': True,
            'dbname': self.db_name,
            'user': self.user,
            'pwd': self.pwd,
            'host': self.host,
            'port': self.pg_port
        }

        delay = 0 + sec
        for model_name in self.models_list:
            kwargs['model_name'] = model_name
            encrypted_kwargs = self.encrypt_data(json.dumps(kwargs), secret_key)
            self.queue.enqueue_in(timedelta(seconds=delay), train_model, encrypted_kwargs, secret_key)
            delay += sec
        self.queue.enqueue_in(timedelta(seconds=delay), self.terminate_rq_worker)



    @staticmethod
    def terminate_rq_worker():
        for process in psutil.process_iter(['pid', 'name']):
            if process.info['name'] == 'rq':
                rq_worker_pid = process.pid
                rq_worker = psutil.Process(rq_worker_pid)
                rq_worker.terminate()

    @staticmethod
    def is_rq_worker_running():
        for process in psutil.process_iter(['pid', 'name']):
            if process.info['name'] == 'rq':
                return True
        return False

    @staticmethod
    def encrypt_data(data, key):
        cipher = Fernet(key)
        return cipher.encrypt(data.encode())
